/*
** $Header: /u/spert/src/intvec/nv/RCS/vecele.c,v 1.4 1996/01/04 23:25:40 davidj Exp $
**
** Element wise vector operations - the quick and easy way
*/

#include <intv-config.h>
#include <stddef.h>
#include <intv-defs.h>
#include <intv-scansrc.h>

/***********************************/
/*** one input scalar/vector ops ***/
/***********************************/

#ifdef DECL_nv_copy_i8_vi8
void nv_copy_i8_vi8(size_t len, intv_int8_t from, intv_int8_t* to);
#endif

#ifdef DEFN_nv_copy_i8_vi8
void
nv_copy_i8_vi8(size_t len, intv_int8_t from, intv_int8_t* to)
{
    size_t i;

    for (i=0; i<len; i++)
	*to++ = from;
}
#endif

/****************************************************************/

#ifdef DECL_nv_copy_i16_vi16
void nv_copy_i16_vi16(size_t len, intv_int16_t from, intv_int16_t* to);
#endif

#ifdef DEFN_nv_copy_i16_vi16
void
nv_copy_i16_vi16(size_t len, intv_int16_t from, intv_int16_t* to)
{
    size_t i;

    for (i=0; i<len; i++)
	*to++ = from;
}
#endif

/****************************************************************/

#ifdef DECL_nv_copy_i32_vi32
void nv_copy_i32_vi32(size_t len, intv_int32_t from, intv_int32_t* to);
#endif

#ifdef DEFN_nv_copy_i32_vi32
void
nv_copy_i32_vi32(size_t len, intv_int32_t from, intv_int32_t* to)
{
    size_t i;

    for (i=0; i<len; i++)
	*to++ = from;
}
#endif

/***********************************/
/*** One input vector/vector ops ***/
/***********************************/

#ifdef DECL_nv_copy_vi8_vi8
void nv_copy_vi8_vi8(size_t len, const intv_int8_t* from, intv_int8_t* to);
#endif

#ifdef DEFN_nv_copy_vi8_vi8
void
nv_copy_vi8_vi8(size_t len, const intv_int8_t* from, intv_int8_t* to)
{
    size_t i;

    for (i=0; i<len; i++)
	*to++ = *from++;
}
#endif

/****************************************************************/

#ifdef DECL_nv_copy_vi16_vi16
void nv_copy_vi16_vi16(size_t len, const intv_int16_t* from, intv_int16_t* to);
#endif

#ifdef DEFN_nv_copy_vi16_vi16
void
nv_copy_vi16_vi16(size_t len, const intv_int16_t* from, intv_int16_t* to)
{
    size_t i;

    for (i=0; i<len; i++)
	*to++ = *from++;
}
#endif

/****************************************************************/

#ifdef DECL_nv_copy_vi32_vi32
void nv_copy_vi32_vi32(size_t len, const intv_int32_t* from, intv_int32_t* to);
#endif

#ifdef DEFN_nv_copy_vi32_vi32
void
nv_copy_vi32_vi32(size_t len, const intv_int32_t* from, intv_int32_t* to)
{
    size_t i;

    for (i=0; i<len; i++)
	*to++ = *from++;
}
#endif

/****************************************************************/

#ifdef DECL_nv_conv_vi16_vi32
void nv_conv_vi16_vi32(size_t len, const intv_int16_t* from, intv_int32_t* to);
#endif

#ifdef DEFN_nv_conv_vi16_vi32
void
nv_conv_vi16_vi32(size_t len, const intv_int16_t* from, intv_int32_t* to)
{
    size_t i;

    for (i=0; i<len; i++)
	*to++ = *from++;
}
#endif

/****************************************************************/

#ifdef DECL_nv_abs_vi32_vi32
void nv_abs_vi32_vi32(size_t len, const intv_int32_t* from, intv_int32_t* to);
#endif

#ifdef DEFN_nv_abs_vi32_vi32
void
nv_abs_vi32_vi32(size_t len, const intv_int32_t* from, intv_int32_t* to)
{
    size_t i;

    for (i=0; i<len; i++)
	*to++ = abs(*from++);
}
#endif

/****************************************************************/

#ifdef DECL_nv_satabs_vi32_vi32
void nv_satabs_vi32_vi32(size_t len, const intv_int32_t* from,
			 intv_int32_t* to);
#endif

#ifdef DEFN_nv_satabs_vi32_vi32
void
nv_satabs_vi32_vi32(size_t len, const intv_int32_t* from, intv_int32_t* to)
{
    size_t i;

    for (i=0; i<len; i++)
	*to++ = satabs_i32_i32(*from++);
}
#endif

/****************************************************************/

#ifdef DECL_nv_neg_vi32_vi32
void nv_neg_vi32_vi32(size_t len, const intv_int32_t* from, intv_int32_t* to);
#endif

#ifdef DEFN_nv_neg_vi32_vi32
void
nv_neg_vi32_vi32(size_t len, const intv_int32_t* from, intv_int32_t* to)
{
    size_t i;

    for (i=0; i<len; i++)
	*to++ = -(*from++);
}
#endif

/****************************************************************/

#ifdef DECL_nv_satneg_vi32_vi32
void nv_satneg_vi32_vi32(size_t len, const intv_int32_t* from,
			 intv_int32_t* to);
#endif

#ifdef DEFN_nv_satneg_vi32_vi32
void
nv_satneg_vi32_vi32(size_t len, const intv_int32_t* from, intv_int32_t* to)
{
    size_t i;

    for (i=0; i<len; i++)
	*to++ = satneg_i32_i32(*from++);
}
#endif

/****************************************************************/

#ifdef DECL_nv_swapb_vi16_vi16
void nv_swapb_vi16_vi16(size_t len, const intv_int16_t* from,
			intv_int16_t* to);
#endif

#ifdef DEFN_nv_swapb_vi16_vi16
void
nv_swapb_vi16_vi16(size_t len, const intv_int16_t* from, intv_int16_t* to)
{
    size_t i;

    for (i=0; i<len; i++)
	*to++ = swapb_i16_i16(*from++);
}
#endif

/****************************************************************/

#ifdef DECL_nv_swapb_vi32_vi32
void nv_swapb_vi32_vi32(size_t len, const intv_int32_t* from,
			intv_int32_t* to);
#endif

#ifdef DEFN_nv_swapb_vi32_vi32
void
nv_swapb_vi32_vi32(size_t len, const intv_int32_t* from, intv_int32_t* to)
{
    size_t i;

    for (i=0; i<len; i++)
	*to++ = swapb_i32_i32(*from++);
}
#endif

/****************************************************************/

#ifdef DECL_nv_countlz_vi32_vi32
void nv_countlz_vi32_vi32(size_t len, const intv_int32_t* in,
			  intv_int32_t* res);
#endif

#ifdef DEFN_nv_countlz_vi32_vi32
void
nv_countlz_vi32_vi32(size_t len, const intv_int32_t* in, intv_int32_t* res)
{
    int i;

    for (i=0; i<len; i++)
    {
	*res++ = countlz_i32_i32(*in++);
    }
}
#endif


/***********************************/
/*** Two input vector/scalar ops ***/
/***********************************/

#ifdef DECL_nv_add_vi32i32_vi32
void nv_add_vi32i32_vi32(size_t len, const intv_int32_t* a, intv_int32_t b,
			 intv_int32_t* res);
#endif

#ifdef DEFN_nv_add_vi32i32_vi32
void
nv_add_vi32i32_vi32(size_t len, const intv_int32_t* a, intv_int32_t b,
		    intv_int32_t* res)
{
    int i;

    for (i=0; i<len; i++)
    {
        *res++ = (*a++) + b;
    }
}
#endif /* DEFN_nv_add_vi32i32_vi32 */

/****************************************************************/

#ifdef DECL_nv_satadd_vi32i32_vi32
void nv_satadd_vi32i32_vi32(size_t len, const intv_int32_t* a, intv_int32_t b,
			 intv_int32_t* res);
#endif

#ifdef DEFN_nv_satadd_vi32i32_vi32
void
nv_satadd_vi32i32_vi32(size_t len, const intv_int32_t* a, intv_int32_t b,
		    intv_int32_t* res)
{
    int i;

    for (i=0; i<len; i++)
    {
        *res++ = satadd_i32i32_i32(*a++, b);
    }
}
#endif /* DEFN_nv_satadd_vi32i32_vi32 */

/****************************************************************/

#ifdef DECL_nv_sub_vi32i32_vi32
void nv_sub_vi32i32_vi32(size_t len, const intv_int32_t* a, intv_int32_t b,
			 intv_int32_t* res);
#endif

#ifdef DEFN_nv_sub_vi32i32_vi32
void
nv_sub_vi32i32_vi32(size_t len, const intv_int32_t* a, intv_int32_t b,
		    intv_int32_t* res)
{
    int i;

    for (i=0; i<len; i++)
    {
        *res++ = (*a++) - b;
    }
}
#endif /* DEFN_nv_sub_vi32i32_vi32 */

/****************************************************************/

#ifdef DECL_nv_satsub_vi32i32_vi32
void nv_satsub_vi32i32_vi32(size_t len, const intv_int32_t* a, intv_int32_t b,
			 intv_int32_t* res);
#endif

#ifdef DEFN_nv_satsub_vi32i32_vi32
void
nv_satsub_vi32i32_vi32(size_t len, const intv_int32_t* a, intv_int32_t b,
		    intv_int32_t* res)
{
    int i;

    for (i=0; i<len; i++)
    {
        *res++ = satsub_i32i32_i32(*a++, b);
    }
}
#endif /* DEFN_nv_satsub_vi32i32_vi32 */

/****************************************************************/

#ifdef DECL_nv_mul_vi32i32_vi32
void nv_mul_vi32i32_vi32(size_t len, const intv_int32_t* a, intv_int32_t b,
			 intv_int32_t* res);
#endif

#ifdef DEFN_nv_mul_vi32i32_vi32
void
nv_mul_vi32i32_vi32(size_t len, const intv_int32_t* a, intv_int32_t b,
		    intv_int32_t* res)
{
    int i;

    for (i=0; i<len; i++)
    {
        *res++ = (*a++) * b;
    }
}
#endif /* DEFN_nv_mul_vi32i32_vi32 */

/****************************************************************/

#ifdef DECL_nv_div_vi32i32_vi32
void nv_div_vi32i32_vi32(size_t len, const intv_int32_t* a, intv_int32_t b,
			 intv_int32_t* res);
#endif

#ifdef DEFN_nv_div_vi32i32_vi32
void
nv_div_vi32i32_vi32(size_t len, const intv_int32_t* a, intv_int32_t b,
		    intv_int32_t* res)
{
    int i;

    for (i=0; i<len; i++)
    {
        *res++ = (*a++) / b;
    }
}
#endif /* DEFN_nv_div_vi32i32_vi32 */

/***********************************/
/*** Two input scalar/vector ops ***/
/***********************************/

#ifdef DECL_nv_sub_i32vi32_vi32
void nv_sub_i32vi32_vi32(size_t len, intv_int32_t a, const intv_int32_t* b,
			 intv_int32_t* res);
#endif

#ifdef DEFN_nv_sub_i32vi32_vi32
void
nv_sub_i32vi32_vi32(size_t len, intv_int32_t a, const intv_int32_t *b,
		    intv_int32_t* res)
{
    int i;

    for (i=0; i<len; i++)
    {
        *res++ = a - *b++;
    }
}
#endif /* DEFN_nv_sub_i32vi32_vi32 */

/****************************************************************/

#ifdef DECL_nv_satsub_i32vi32_vi32
void nv_satsub_i32vi32_vi32(size_t len, intv_int32_t a, const intv_int32_t* b,
			 intv_int32_t* res);
#endif

#ifdef DEFN_nv_satsub_i32vi32_vi32
void
nv_satsub_i32vi32_vi32(size_t len, intv_int32_t a, const intv_int32_t *b,
		    intv_int32_t* res)
{
    int i;

    for (i=0; i<len; i++)
    {
        *res++ = satsub_i32i32_i32(a, *b++);
    }
}
#endif /* DEFN_nv_satsub_i32vi32_vi32 */

/****************************************************************/

#ifdef DECL_nv_div_i32vi32_vi32
void nv_div_i32vi32_vi32(size_t len, intv_int32_t a, const intv_int32_t* b,
			 intv_int32_t* res);
#endif

#ifdef DEFN_nv_div_i32vi32_vi32
void
nv_div_i32vi32_vi32(size_t len, intv_int32_t a, const intv_int32_t *b,
		    intv_int32_t* res)
{
    int i;

    for (i=0; i<len; i++)
    {
        *res++ = a - *b++;
    }
}
#endif /* DEFN_nv_div_i32vi32_vi32 */

/***********************************/
/*** Two input vector/vector ops ***/
/***********************************/

#ifdef DECL_nv_add_vi32vi32_vi32
void nv_add_vi32vi32_vi32(size_t len, const intv_int32_t* a,
			  const intv_int32_t* b, intv_int32_t* res);
#endif

#ifdef DEFN_nv_add_vi32vi32_vi32
void
nv_add_vi32vi32_vi32(size_t len, const intv_int32_t* a,
		     const intv_int32_t* b, intv_int32_t* res)
{
    int i;

    for (i=0; i<len; i++)
    {
        *res++ = (*a++) + (*b++);
    }
}
#endif /* DEFN_nv_add_vi32vi32_vi32 */

/****************************************************************/

#ifdef DECL_nv_satadd_vi32vi32_vi32
void nv_satadd_vi32vi32_vi32(size_t len, const intv_int32_t* a,
			  const intv_int32_t* b, intv_int32_t* res);
#endif

#ifdef DEFN_nv_satadd_vi32vi32_vi32
void
nv_satadd_vi32vi32_vi32(size_t len, const intv_int32_t* a,
		     const intv_int32_t* b, intv_int32_t* res)
{
    int i;

    for (i=0; i<len; i++)
    {
        *res++ = satadd_i32i32_i32(*a++, *b++);
    }
}
#endif /* DEFN_nv_satadd_vi32vi32_vi32 */

/****************************************************************/

#ifdef DECL_nv_sub_vi32vi32_vi32
void nv_sub_vi32vi32_vi32(size_t len, const intv_int32_t* a,
			  const intv_int32_t* b, intv_int32_t* res);
#endif

#ifdef DEFN_nv_sub_vi32vi32_vi32
void
nv_sub_vi32vi32_vi32(size_t len, const intv_int32_t* a,
		     const intv_int32_t* b, intv_int32_t* res)
{
    int i;

    for (i=0; i<len; i++)
    {
        *res++ = (*a++) - (*b++);
    }
}
#endif /* DEFN_nv_sub_vi32vi32_vi32 */

/****************************************************************/

#ifdef DECL_nv_satsub_vi32vi32_vi32
void nv_satsub_vi32vi32_vi32(size_t len, const intv_int32_t* a,
			  const intv_int32_t* b, intv_int32_t* res);
#endif

#ifdef DEFN_nv_satsub_vi32vi32_vi32
void
nv_satsub_vi32vi32_vi32(size_t len, const intv_int32_t* a,
		     const intv_int32_t* b, intv_int32_t* res)
{
    int i;

    for (i=0; i<len; i++)
    {
        *res++ = satsub_i32i32_i32(*a++, *b++);
    }
}
#endif /* DEFN_nv_satsub_vi32vi32_vi32 */

/****************************************************************/

#ifdef DECL_nv_mul_vi32vi32_vi32
void nv_mul_vi32vi32_vi32(size_t len, const intv_int32_t* a,
			  const intv_int32_t* b, intv_int32_t* res);
#endif

#ifdef DEFN_nv_mul_vi32vi32_vi32
void
nv_mul_vi32vi32_vi32(size_t len, const intv_int32_t* a,
		     const intv_int32_t* b, intv_int32_t* res)
{
    int i;

    for (i=0; i<len; i++)
    {
        *res++ = (*a++) * (*b++);
    }
}
#endif /* DEFN_nv_mul_vi32vi32_vi32 */

/****************************************************************/

#ifdef DECL_nv_div_vi32vi32_vi32
void nv_div_vi32vi32_vi32(size_t len, const intv_int32_t* a,
			  const intv_int32_t* b, intv_int32_t* res);
#endif

#ifdef DEFN_nv_div_vi32vi32_vi32
void
nv_div_vi32vi32_vi32(size_t len, const intv_int32_t* a,
		     const intv_int32_t* b, intv_int32_t* res)
{
    int i;

    for (i=0; i<len; i++)
    {
        *res++ = (*a++) / (*b++);
    }
}
#endif /* DEFN_nv_div_vi32vi32_vi32 */



