/*
   $Header: /u/spert/src/intvec/testsuite/RCS/copy_i_vi_test.c,v 1.2 1996/10/23 03:10:40 davidj Exp $

   Test element wise vector routines of the form op_vi16_vi32.
*/

#include <stddef.h>
#include <stdlib.h>

#include "intvec_test.h"

typedef void (*rtn8_t)(size_t len, const intv_int8_t in,
		      intv_int8_t* res);

typedef void (*rtn16_t)(size_t len, const intv_int16_t in,
		      intv_int16_t* res);

typedef void (*rtn32_t)(size_t len, const intv_int32_t in,
		      intv_int32_t* res);

/* Performs one test given input values and expected output. */

void
onetest_i8(rtn8_t rtn, size_t len, intv_int8_t in)
{
    size_t i;

    /* Create padded copies of inputs. */
    intv_int8_t* c_res = rtst_padvec_new_vi8(len);

    /* Run routine. */
    rtn(len, in, c_res);

    /* Check result. */
    for (i=0; i<len; i++)
	rtst_assert(in==c_res[i]);

    /* Free and check padding. */
    rtst_padvec_del_vi8(c_res);
}

void
onetest_i16(rtn16_t rtn, size_t len, intv_int16_t in)
{
    size_t i;

    /* Create padded copies of inputs. */
    intv_int16_t* c_res = rtst_padvec_new_vi16(len);

    /* Run routine. */
    rtn(len, in, c_res);

    /* Check result. */
    for (i=0; i<len; i++)
	rtst_assert(in==c_res[i]);

    /* Free and check padding. */
    rtst_padvec_del_vi16(c_res);
}

void
onetest_i32(rtn32_t rtn, size_t len, intv_int32_t in)
{
    size_t i;

    /* Create padded space for result. */
    intv_int32_t* c_res = rtst_padvec_new_vi32(len);

    /* Run routine. */
    rtn(len, in, c_res);

    /* Check result. */
    for (i=0; i<len; i++)
	rtst_assert(in==c_res[i]);

    /* Free and check padding. */
    rtst_padvec_del_vi32(c_res);
}

void
randomtest_i8(rtn8_t rtn)
{
    const int numtests = rtst_numtests;
    const size_t maxlen = rtst_sizetests;

    int i;
    for (i=0; i<numtests; i++)
    {
        size_t len = (size_t) rtst_urand_i32i32_i32(1, maxlen);
	intv_int8_t in;

        in = rtst_urand_i8i8_i8(-0x80, 0x7f);
        rtst_log("len=%d\n", len);
        onetest_i8(rtn, len, in);
    }
}

void
randomtest_i16(rtn16_t rtn)
{
    const int numtests = rtst_numtests;
    const size_t maxlen = rtst_sizetests;

    int i;
    for (i=0; i<numtests; i++)
    {
        size_t len = (size_t) rtst_urand_i32i32_i32(1, maxlen);
	intv_int16_t in;

        in = rtst_badrand_i16();
        rtst_log("len=%d\n", len);
        onetest_i16(rtn, len, in);
    }
}

void
randomtest_i32(rtn32_t rtn)
{
    const int numtests = rtst_numtests;
    const size_t maxlen = rtst_sizetests;

    int i;
    for (i=0; i<numtests; i++)
    {
        size_t len = (size_t) rtst_urand_i32i32_i32(1, maxlen);
	intv_int32_t in;

        in = rtst_badrand_i32();
        rtst_log("len=%d\n", len);
        onetest_i32(rtn, len, in);
    }
}

void
test_i8()
{
    rtst_start("copy_i8_vi8");
    rtst_log("testing nv.\n");
    onetest_i8(nv_copy_i8_vi8, 1, 0);
    onetest_i8(nv_copy_i8_vi8, 3, 1);
    onetest_i8(nv_copy_i8_vi8, 10, -13);
    randomtest_i8(nv_copy_i8_vi8);
#ifdef CHECK_TOR
    rtst_log("testing tor.\n");
    onetest_i8(tor_copy_i8_vi8, 1, 0);
    onetest_i8(tor_copy_i8_vi8, 3, 1);
    onetest_i8(tor_copy_i8_vi8, 10, -13);
    randomtest_i8(tor_copy_i8_vi8);
#endif
    rtst_passed();
}

void
test_i16()
{
    rtst_start("copy_i16_vi16");
    rtst_log("testing nv.\n");
    onetest_i16(nv_copy_i16_vi16, 1, 0);
    onetest_i16(nv_copy_i16_vi16, 3, 1);
    onetest_i16(nv_copy_i16_vi16, 10, -13);
    randomtest_i16(nv_copy_i16_vi16);
#ifdef CHECK_TOR
    rtst_log("testing tor.\n");
    onetest_i16(tor_copy_i16_vi16, 1, 0);
    onetest_i16(tor_copy_i16_vi16, 3, 1);
    onetest_i16(tor_copy_i16_vi16, 10, -13);
    randomtest_i16(tor_copy_i16_vi16);
#endif
    rtst_passed();
}

void
test_i32()
{
    rtst_start("copy_i32_vi32");
    rtst_log("testing nv.\n");
    onetest_i32(nv_copy_i32_vi32, 1, 0);
    onetest_i32(nv_copy_i32_vi32, 3, 1);
    onetest_i32(nv_copy_i32_vi32, 10, -13);
    randomtest_i32(nv_copy_i32_vi32);
#ifdef CHECK_TOR
    rtst_log("testing tor.\n");
    onetest_i32(tor_copy_i32_vi32, 1, 0);
    onetest_i32(tor_copy_i32_vi32, 3, 1);
    onetest_i32(tor_copy_i32_vi32, 10, -13);
    randomtest_i32(tor_copy_i32_vi32);
#endif
    rtst_passed();
}



int
main(int argc, char *argv[])
{
    rtst_args(argc, argv);

    test_i8();
    test_i16();
    test_i32();

    rtst_exit();

    return 0;
}
