/*
** $Id: countlz_gi32_gi32.S,v 1.1 1995/10/10 18:49:38 davidj Exp $
**
** jhauser - Summer 1994
** modified and included in library by davidj - Thu Nov 17 21:23:57 1994
** pulled int intvec by davidj - Tue Oct 10 11:21:20 1995
*/

#include <tor/tordef.h>
#include "m_regdef.h"

	.globl tor_countlz_gi32_gi32

#ifdef DECL_tor_countlz_gi32_gi32
void tor_countlz_gi32_gi32(void);
#endif

#ifdef DEFN_tor_countlz_gi32_gi32
#define vA va0			/* Input				*/
#define vCount vz0		/* Output count of leading zeros	*/

#define vCount16 vt0		/* vCount & 0x10			*/
#define vCount8 vt1		/* vCount & 0x18			*/
#define vLowIndex vt2		/* Index into low zeros-count table	*/
#define vHighIndex vt3		/* Index into high zeros-count table	*/

#define temp0 v0
#define temp1 v1
#define temp2 t4
#define temp3 t5
#define config0 t6
#define config1 t7
#define config2 t8

	.ent tor_countlz_gi32_gi32

/*
-------------------------------------------------------------------------------
Routine:  tor_countlz_gi32_gi32

Input:
	vlr
	va0	vA

Output:
	vv2	vCount

Not preserved:
	vtt
	vv3
	va0--va3
	vt0--vt5
	v0--v1
	t4--t9

Function:

An element-wise ``count leading zeros'' function is performed.  For each of the
32-bit unsigned integer values in vector vA, the number of zero bits preceding
the most significant 1 bit is returned in vector vCount.  The vlr register
controls the elements of vCount actually updated.  For zero inputs, the number
of leading zero bits is given as 32.

-------------------------------------------------------------------------------
*/

tor_countlz_gi32_gi32:

/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
|	va0	vA
|	vt0	vCount16
|	vt1	vCount8
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
If vA has at least 16 leading zeros, shift up by 16.  Set vCount16 to either 0
or 16.
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
	li temp0,(1<<16)
	sltu.vs vCount16,vA,temp0
	li temp1,4
	sllv.vs vCount16,vCount16,temp1
	sllv.vv vA,vA,vCount16

/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
If vA has at least 8 leading zeros, shift up by 8.  Set vCount8 to either 0
or 8, and add in vCount16.  (vCount8 can then be any of 0, 8, 16, or 24.)
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
	li temp2,(1<<24)
	sltu.vs vCount8,vA,temp2
	li temp3,3
	sllv.vs vCount8,vCount8,temp3
	sllv.vv vA,vA,vCount8
	addu.vv vCount8,vCount16,vCount8

/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
|	va0	vA
|	vv2	vCount
|	vt1	vCount8
|	vt2	vLowIndex
|	vt3	vHighIndex
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
Assuming vA is nonzero and that the first bit of vA is zero, load vCount with
the number of leading bits of vA using look-up tables.  vCount will thus be
calculated to be between 1 and 7.  The _low_ table assumes the first 4 bits of
vA are zeros.  The _high_ table assumes only that at least the first bit of vA
is zero, and overrides the low table unless bits 30..28 of vA are all zeros.
vCount will be meaningless if vA is either zero or has no leading zero bits.
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
	li config0,(FX_SHLZA|FX_A_AND_B|(23*FX_SHLAM)|FX_NOROUND|FX_CLIP_PASS)
	li config1,(FX_SHLV|(28*FX_SHLAM)|FX_NOROUND|FX_CLIP_PASS|FX_GEZ)
	li temp0,(7<<25)
	fxadd.vs vLowIndex,vA,temp0,config0
	li temp1,0x76554444
	fxadd.sv vCount,temp1,vLowIndex,config1
	li temp2,(7<<28)
	addu config2,config0,(3*FX_SHLAM)
	fxadd.vs vHighIndex,vA,temp2,config2
	li temp3,0xF3221111
	fxadd.sv vCount,temp3,vHighIndex,config1

/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
|	va0	vA
|	vv2	vCount
|	vt1	vCount8
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
If vA has no leading zero bits, load vCount with 0.  Else if vA is zero, load
vCount with 8.  (vA can be zero now only if it was zero at the start.
Count-leading-zeros of zero is defined to be 32.)
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
	cmvltz.vv vCount,vA,vzero
	li temp0,8
	cmveqz.vs vCount,vA,temp0

/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
Combine vCount with vCount8.
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
	addu.vv vCount,vCount8,vCount

/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
Return.
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
	j ra

	.end tor_countlz_gi32_gi32

#endif
